@@ -17,14 +17,17 @@ from .aswarm.util import function_to_jsonschema
 from .mcp_server import (
     CallTool,
     CallToolResult,
+    Close,
     ListTools,
     ListToolsResult,
-    MCPTool,
     MCPImageContent,
     MCPTextContent,
+    MCPTool,
+    NamedMCPServer,
 )
+from .memory import Memory, NoMemory
 from .messages import ChatMessage, ChatHistory, StructuredOutput
-from .model_client import default_model_client, ModelClient
+from .model import default_model, Model
 from .util import is_user_confirmed
 
 
@@ -217,9 +220,10 @@ class ChatAgent(BaseAgent):
         name: str = "",
         system: str = "",
         tools: list[Callable] | None = None,
-        mcp_servers: list[str] | None = None,
+        mcp_servers: list[NamedMCPServer] | None = None,
         mcp_server_agent_type: str = "mcp_server",
-        client: ModelClient = default_model_client,
+        model: Model = default_model,
+        memory: Memory | None = None,
         timeout: float = 300,
     ):
         super().__init__(timeout=timeout)
@@ -227,13 +231,14 @@ class ChatAgent(BaseAgent):
         self._name: str = name
         self._system: str = system
         self._tools: list[Callable] = tools or []
-        self._mcp_servers: list[str] = mcp_servers or []
+        self._mcp_servers: list[NamedMCPServer] = mcp_servers or []
         self._mcp_server_agent_type: str = mcp_server_agent_type
-        self._client: ModelClient = client
+        self._model: Model = model
 
-        self._swarm_client: Swarm = Swarm(self.client)
+        self._swarm_client: Swarm = Swarm(self.model)
         self._swarm_agent: SwarmAgent | None = None
 
+        self._memory: Memory = memory or NoMemory()
         self._history: ChatHistory = ChatHistory(messages=[])
 
     @property
@@ -254,7 +259,7 @@ class ChatAgent(BaseAgent):
         return self._tools
 
     @property
-    def mcp_servers(self) -> list[str]:
+    def mcp_servers(self) -> list[NamedMCPServer]:
         return self._mcp_servers
 
     @property
@@ -262,8 +267,22 @@ class ChatAgent(BaseAgent):
         return self._mcp_server_agent_type
 
     @property
-    def client(self) -> ModelClient:
-        return self._client
+    def model(self) -> Model:
+        return self._model
+
+    @property
+    def memory(self) -> Memory:
+        return self._memory
+
+    async def stopped(self) -> None:
+        for server in self.mcp_servers:
+            if server.connect is not None:
+                # Close the MCP server if it's auto-connected by the current agent.
+                await self.channel.publish(
+                    Address(name=self.mcp_server_agent_type, id=server.name),
+                    Close().encode(),
+                    request=False,
+                )
 
     def get_swarm_client(self, extensions: dict) -> Swarm:
         """Get the swarm client with the given message extensions.
@@ -273,13 +292,13 @@ class ChatAgent(BaseAgent):
         model_id = extensions.get("model_id", "")
         if model_id:
             # We assume that non-empty model ID indicates the use of a dynamic model client.
-            client = ModelClient(
-                model=model_id,
+            model = Model(
+                id=model_id,
                 base_url=extensions.get("model_base_url", ""),
                 api_key=extensions.get("model_api_key", ""),
                 api_version=extensions.get("model_api_version", ""),
             )
-            return Swarm(client)
+            return Swarm(model)
 
         return self._swarm_client
 
@@ -299,7 +318,7 @@ class ChatAgent(BaseAgent):
 
             self._swarm_agent = SwarmAgent(
                 name=self.name,
-                model=self.client.model,
+                model=self.model.id,
                 instructions=self.system,
                 functions=[wrap_error(t) for t in tools],
             )
@@ -307,6 +326,7 @@ class ChatAgent(BaseAgent):
 
     async def agent(self, agent_type: str) -> AsyncIterator[ChatMessage]:
         """The candidate agent to delegate the conversation to."""
+        # TODO: Handle memory?
         async for chunk in Delegate(self, agent_type).handle(self._history):
             yield chunk
 
@@ -314,6 +334,7 @@ class ChatAgent(BaseAgent):
     async def handle_history(
         self, msg: ChatHistory, ctx: Context
     ) -> AsyncIterator[ChatMessage]:
+        # TODO: Handle memory?
         response = self._handle_history(msg)
         async for resp in response:
             yield resp
@@ -322,15 +343,25 @@ class ChatAgent(BaseAgent):
     async def handle_message(
         self, msg: ChatMessage, ctx: Context
     ) -> AsyncIterator[ChatMessage]:
-        history = ChatHistory(messages=[msg])
+        existing = await self.memory.get_items()
+        history = ChatHistory(messages=existing + [msg])
+
         response = self._handle_history(history)
+        full_content = ""
         async for resp in response:
             yield resp
+            full_content += resp.content
+
+        await self.memory.add_items(
+            msg,  # input item
+            ChatMessage(role="assistant", content=full_content),  # output item
+        )
 
     @handler
     async def handle_structured_output(
         self, msg: StructuredOutput, ctx: Context
     ) -> AsyncIterator[ChatMessage]:
+        # TODO: Handle memory?
         match msg.input:
             case ChatMessage():
                 history = ChatHistory(messages=[msg.input])
@@ -342,13 +373,13 @@ class ChatAgent(BaseAgent):
                 async for resp in response:
                     yield resp
 
-    async def _get_mcp_tools(self, mcp_servers: list[str]) -> list[Callable]:
+    async def _get_mcp_tools(self, mcp_servers: list[NamedMCPServer]) -> list[Callable]:
         all_tools = []
 
         for server in mcp_servers:
             raw_result = await self.channel.publish(
-                Address(name=self.mcp_server_agent_type, id=server),
-                ListTools().encode(),
+                Address(name=self.mcp_server_agent_type, id=server.name),
+                ListTools(connect=server.connect).encode(),
                 request=True,
                 timeout=10,
             )
@@ -359,17 +390,18 @@ class ChatAgent(BaseAgent):
 
         return all_tools
 
-    def _to_function_tool(self, server: str, t: MCPTool) -> Callable:
+    def _to_function_tool(self, server: NamedMCPServer, t: MCPTool) -> Callable:
         async def tool(**kwargs) -> Any:
             # Validate the input against the schema
             jsonschema.validate(instance=kwargs, schema=t.inputSchema)
 
             # Actually call the tool.
             raw_result = await self.channel.publish(
-                Address(name=self.mcp_server_agent_type, id=server),
+                Address(name=self.mcp_server_agent_type, id=server.name),
                 CallTool(
                     name=t.name,
                     arguments=kwargs,
+                    connect=server.connect,
                 ).encode(),
                 request=True,
                 timeout=10,
