@@ -5,8 +5,8 @@ from .agent import (
     handler,
 )
 from .discovery import DiscoveryQuery, DiscoveryReply
-from .logger import logger, set_stderr_logger
-from .messages import Message, GenericMessage, SetReplyInfo, StopIteration
+from .logger import logger, init_logger
+from .messages import GenericMessage, Message, SetReplyInfo, StopIteration
 from .runtime import BaseRuntime, BaseChannel, QueueSubscriptionIterator
 from .types import (
     Address,
@@ -16,9 +16,7 @@ from .types import (
     Channel,
     MessageHeader,
     new,
-    NO_REPLY,
     RawMessage,
     Reply,
     Subscription,
 )
-from .util import idle_loop
