@@ -95,13 +95,6 @@ class Reply(BaseModel):
     )
 
 
-# A sentinel reply object that indicates no reply will be sent.
-#
-# This is mainly used by the orchestration agents who delegate the reply
-# handling to other agents.
-NO_REPLY = Reply(address=Address(name="", id=""), stream=False)
-
-
 class MessageHeader(BaseModel):
     type: str = Field(..., description="Message type name.")
     content_type: str = Field(
@@ -324,6 +317,11 @@ class Runtime(abc.ABC):
     async def stop(self) -> None:
         pass
 
+    @abc.abstractmethod
+    async def wait_for_shutdown(self, timeout: float | None = None) -> None:
+        """Wait for the shutdown event with a timeout."""
+        pass
+
     @abc.abstractmethod
     async def register(self, spec: AgentSpec) -> None:
         pass
